"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DiscreteLegend = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _utils = require("./utils.js");

var _covutils = require("covutils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

var DEFAULT_TEMPLATE_ID = 'template-coverage-parameter-discrete-legend';
var DEFAULT_TEMPLATE = "<template id=\"".concat(DEFAULT_TEMPLATE_ID, "\">\n  <div class=\"leaflet-coverage-control legend discrete-legend\">\n    <div class=\"legend-title-container\"><strong class=\"legend-title\"></strong></div>\n    <div class=\"legend-palette discrete-legend-palette\"></div>\n  </div>\n</template>");
/**
 * Displays a discrete palette legend for the parameter displayed by the given
 * Coverage layer. Supports category parameters only at the moment.
 * 
 * @example <caption>Coverage data layer</caption>
 * new C.DiscreteLegend(covLayer).addTo(map)
 * // changing the palette of the layer automatically updates the legend 
 * covLayer.palette = C.discretePalette(['red', 'blue'])
 * 
 * @example <caption>Fake layer</caption>
 * var fakeLayer = {
 *   parameter: {
 *     observedProperty: {
 *       label: { en: 'Land cover' },
 *       categories: [{
 *         label: { en: 'Land' }
 *       }, {
 *         label: { en: 'Water' }
 *       }]
 *     }
 *   },
 *   palette: C.directPalette(['gray', 'blue']) // CSS colors in category order
 * }
 * var legend = new C.DiscreteLegend(fakeLayer).addTo(map)
 * 
 * // change the palette and trigger a manual update
 * fakeLayer.palette = C.discretePalette(['red', 'blue'])
 * legend.update()
 * 
 * @extends {L.Control}
 */

var DiscreteLegend = /*#__PURE__*/function (_L$Control) {
  _inherits(DiscreteLegend, _L$Control);

  var _super = _createSuper(DiscreteLegend);

  /**
   * Creates a discrete legend control.
   * 
   * @param {object} covLayer 
   *   The coverage data layer, or any object with <code>palette</code>
   *   and <code>parameter</code> properties.
   *   If the object has <code>on</code>/<code>off</code> methods, then the legend will
   *   listen for <code>"paletteChange"</code> events and update itself automatically.
   *   If the layer fires a <code>"remove"</code> event, then the legend will remove itself
   *   from the map. 
   * @param {object} [options] Legend options.
   * @param {string} [options.position='bottomright'] The initial position of the control (see Leaflet docs).
   * @param {string} [options.language] A language tag, indicating the preferred language to use for labels.
   * @param {string} [options.templateId] Uses the HTML element with the given id as template.
   */
  function DiscreteLegend(covLayer) {
    var _this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, DiscreteLegend);

    _this = _super.call(this, {
      position: options.position || 'bottomright'
    });
    _this._covLayer = covLayer;
    _this._templateId = options.templateId || DEFAULT_TEMPLATE_ID;
    _this._language = options.language;

    if (!options.templateId && document.getElementById(DEFAULT_TEMPLATE_ID) === null) {
      (0, _utils.inject)(DEFAULT_TEMPLATE);
    }

    if (covLayer.on) {
      _this._remove = function () {
        return _this.remove();
      };

      _this._update = function () {
        return _this._doUpdate(false);
      };

      covLayer.on('remove', _this._remove);
    }

    return _this;
  }
  /**
   * Triggers a manual update of the legend.
   * 
   * Useful if the supplied coverage data layer is not a real layer
   * and won't fire the necessary events for automatic updates.
   */


  _createClass(DiscreteLegend, [{
    key: "update",
    value: function update() {
      this._doUpdate(true);
    }
  }, {
    key: "_doUpdate",
    value: function _doUpdate(fullUpdate) {
      var el = this._el;

      if (fullUpdate) {
        var _param = this._covLayer.parameter; // if requested language doesn't exist, use the returned one for all other labels

        this._language = (0, _covutils.getLanguageTag)(_param.observedProperty.label, this._language);
        var title = (0, _covutils.getLanguageString)(_param.observedProperty.label, this._language);
        (0, _utils.$$)('.legend-title', el).innerHTML = title;
      }

      var palette = this._covLayer.palette;
      var param = this._covLayer.parameter;
      var html = '';

      for (var i = 0; i < palette.steps; i++) {
        var cat = (0, _covutils.getLanguageString)(param.observedProperty.categories[i].label, this._language);
        html += "\n        <i style=\"background:rgb(".concat(palette.red[i], ", ").concat(palette.green[i], ", ").concat(palette.blue[i], ")\"></i>\n        ").concat(cat, "\n        <br>");
      }

      (0, _utils.$$)('.legend-palette', el).innerHTML = html;
    }
    /**
     * @override
     * @ignore
     */

  }, {
    key: "onAdd",
    value: function onAdd(map) {
      this._map = map;

      if (this._covLayer.on) {
        this._covLayer.on('paletteChange', this._update);
      }

      this._el = (0, _utils.fromTemplate)(this._templateId);
      this.update();
      return this._el;
    }
    /**
     * @override
     * @ignore
     */

  }, {
    key: "onRemove",
    value: function onRemove() {
      if (this._covLayer.off) {
        this._covLayer.off('remove', this._remove);

        this._covLayer.off('paletteChange', this._update);
      }
    }
  }]);

  return DiscreteLegend;
}(_leaflet.default.Control);

exports.DiscreteLegend = DiscreteLegend;