"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VerticalAxis = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _Dropdown = require("./Dropdown.js");

var _EventMixin = require("../util/EventMixin.js");

var _covutils = require("covutils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/**
 * The `change` event, signalling that a different vertical coordinate value has been selected.
 * 
 * @typedef {L.Event} VerticalAxis#change
 * @property {number} vertical The vertical coordinate value that has been selected.
 */

/**
 * Displays a simple vertical coordinate dropdown selector for a coverage data layer.
 * 
 * @example <caption>Coverage data layer</caption>
 * new C.VerticalAxis(covLayer).addTo(map)
 * // Selecting a vertical coordinate automatically sets the 'vertical' property in the layer.
 * // Similarly, when the layer fires an 'axisChange' event with {axis: 'vertical'}
 * // the control reflects that change.
 * 
 * @example <caption>Fake layer</caption>
 * var heights = [0,10,20,50,100,500,1000]
 * var fakeLayer = {
 *   verticalSlices: heights,
 *   vertical: heights[1], // select the second height step initially
 *   crsVerticalAxis: {
 *     name: { 
 *       en: 'Gravity-related height'
 *     },
 *     unit: {
 *       symbol: 'm'
 *     }
 *   }
 * }
 * var verticalAxis = new C.VerticalAxis(fakeLayer).addTo(map)
 * 
 * // change the height and trigger a manual update
 * fakeLayer.vertical = heights[0]
 * verticalAxis.update()
 * 
 * @extends {L.Layer}
 * 
 * @emits {VerticalAxis#change} when a different entry has been selected
 */
var VerticalAxis = /*#__PURE__*/function (_L$Layer) {
  _inherits(VerticalAxis, _L$Layer);

  var _super = _createSuper(VerticalAxis);

  /**
   * Creates a vertical axis control.
   * 
   * @param {object} covLayer 
   *   The coverage data layer, or any object with `verticalSlices`
   *   and `vertical` properties, optionally `crsVerticalAxis` property.
   *   If the object has `on`/`off` methods, then the control will
   *   listen for `axisChange` events with `{axis: 'vertical'}`
   *   and update itself automatically.
   *   If the layer fires a `remove` event, then the control will remove itself
   *   from the map.
   * @param {object} [options] Control options.
   * @param {string} [options.position='topleft'] The initial position of the control (see Leaflet docs).
   * @param {string} [options.title='Vertical'] 
   *   The label to show above the control if `covLayer.crsVerticalAxis.name` is missing.
   * @param {string} [options.templateId] Element ID of an alternative HTML `<template>` element to use. 
   */
  function VerticalAxis(covLayer) {
    var _this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, VerticalAxis);

    _this = _super.call(this);
    _this._templateId = options.templateId;
    _this._covLayer = covLayer;
    _this._title = options.title || 'Vertical';
    _this._position = options.position || 'topleft';

    if (covLayer.on) {
      _this._remove = function () {
        return _this.remove();
      };

      covLayer.on('remove', _this._remove);

      _this._axisListener = function (e) {
        if (e.axis === 'vertical') _this.update();
      };
    }

    return _this;
  }
  /**
   * @ignore
   * @override
   */


  _createClass(VerticalAxis, [{
    key: "onAdd",
    value: function onAdd(map) {
      var _this2 = this;

      this._map = map;

      if (this._covLayer.on) {
        this._covLayer.on('axisChange', this._axisListener);
      }

      var crsVertAxis = this._covLayer.crsVerticalAxis || {};
      var title = crsVertAxis.name ? (0, _covutils.getLanguageString)(crsVertAxis.name) : this._title;
      var unit = (0, _covutils.stringifyUnit)(crsVertAxis.unit);

      if (unit) {
        unit = ' ' + unit;
      }

      var choices = [];
      var vals = this._covLayer.verticalSlices;

      for (var i = 0; i < vals.length; i++) {
        choices.push({
          value: i.toString(),
          label: vals[i] + unit
        });
      }

      this._dropdown = new _Dropdown.Dropdown(choices, {
        templateId: this._templateId,
        position: this._position,
        title: title,
        value: this._getVerticalIndex().toString()
      }).on('change', function (event) {
        var i = parseInt(event.value);
        var val = _this2._covLayer.verticalSlices[i];
        _this2._covLayer.vertical = val;

        _this2.fire('change', {
          vertical: val
        });
      }).addTo(map);
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "onRemove",
    value: function onRemove(map) {
      this._dropdown.remove();

      if (this._covLayer.off) {
        this._covLayer.off('remove', this._remove);

        this._covLayer.off('axisChange', this._axisListener);
      }
    }
  }, {
    key: "_getVerticalIndex",
    value: function _getVerticalIndex() {
      var vals = this._covLayer.verticalSlices;
      var i = vals.indexOf(this._covLayer.vertical);
      return i;
    }
    /**
     * Triggers a manual update of the vertical axis control based on the
     * `vertical` property of the layer.
     * 
     * Useful if the supplied coverage data layer is not a real layer
     * and won't fire the necessary events for automatic updates.
     */

  }, {
    key: "update",
    value: function update() {
      var i = this._getVerticalIndex();

      this._dropdown.value = i.toString();
    }
  }]);

  return VerticalAxis;
}(_leaflet.default.Layer);

exports.VerticalAxis = VerticalAxis;