"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.$ = $;
exports.$$ = $$;
exports.HTML = HTML;
exports.fromTemplate = fromTemplate;
exports.inject = inject;

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Returns the first child element of parent (fall-back to document if not given)
 * matching the given selector.
 * 
 * @private
 */
function $$(selector, parent) {
  if (typeof parent === 'string') {
    parent = $$(parent);
  }

  parent = parent || document;
  return parent.querySelector(selector);
}
/**
 * Returns all child elements of parent (fall-back to document if not given)
 * matching the given selector as an array.
 * 
 * @private
 */


function $(selector, parent) {
  if (typeof parent === 'string') {
    parent = $$(parent);
  }

  parent = parent || document;
  return _toConsumableArray(parent.querySelectorAll(selector));
}
/**
 * Turns an HTML string into a DOM element.
 * The HTML markup must have a single root node not prepended by any whitespace.
 * 
 * @example 
 * var s = '<li>text</li>'
 * var el = C.HTML(s)
 * document.body.appendChild(el)
 * 
 * @private
 */


function HTML(html) {
  var div = document.createElement('div');
  div.innerHTML = html;
  var element = div.firstChild;
  return element;
}
/**
 * Inject HTML and CSS into the DOM.
 * 
 * @param html The html to inject at the end of the body element. Must have a single root node without surrounding whitespace.
 * @param css The CSS styles to inject at the end of the head element.
 * 
 * @private
 */


function inject(html, css) {
  // inject default template and CSS into DOM
  if (html) {
    document.body.appendChild(HTML(html));
  }

  if (css) {
    var style = document.createElement('style');
    style.type = 'text/css';

    if (style.styleSheet) {
      style.styleSheet.cssText = css;
    } else {
      style.appendChild(document.createTextNode(css));
    }

    document.head.appendChild(style);
  }
}
/**
 * @private
 */


function fromTemplate(id) {
  var node = $$('#' + id); // browsers without <template> support don't wrap everything in .content

  if ('content' in node) {
    node = node.content;
  }

  return document.importNode(node, true).children[0];
}