"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CircleMarkerMixin = CircleMarkerMixin;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _palettes = require("./palettes.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/**
 * A mixin that encapsulates the creation, update, and removal
 * of a CircleMarker.
 * 
 * It provides the public methods `bindPopup`, `openPopup`, `closePopup`, and `redraw`.
 * 
 * See {@link Point} and {@link VerticalProfile} for usage.
 * 
 * @param {PointDataLayer} base The base class implementing the {@link PointDataLayer} interface.
 * @return {class} The base class with CircleMarkerMixin.
 */
function CircleMarkerMixin(base) {
  return /*#__PURE__*/function (_base) {
    _inherits(_class, _base);

    var _super = _createSuper(_class);

    function _class() {
      _classCallCheck(this, _class);

      return _super.apply(this, arguments);
    }

    _createClass(_class, [{
      key: "_addMarker",
      value: function _addMarker() {
        var _this = this;

        var val = this.getValue();

        if (val === null && !this.showNoData) {
          return;
        }

        var color = this._getColor(val);

        var _ref = typeof color === 'string' ? (0, _palettes.cssToRGB)(color) : color,
            r = _ref.r,
            g = _ref.g,
            b = _ref.b;

        var latlng = this.getLatLng();
        var strokeBrightness = 0.7;
        this._marker = _leaflet.default.circleMarker(latlng, {
          fillColor: 'rgb(' + r + ',' + g + ',' + b + ')',
          fillOpacity: 1,
          radius: 5,
          stroke: true,
          opacity: 1,
          weight: 1,
          color: 'rgb(' + Math.round(r * strokeBrightness) + ',' + Math.round(g * strokeBrightness) + ',' + Math.round(b * strokeBrightness) + ')'
        }).on('click', function (e) {
          e.coverage = _this.coverage;

          _this.fire('click', e);
        }).addTo(this._map);

        if (this._popup) {
          var _this$_marker;

          (_this$_marker = this._marker).bindPopup.apply(_this$_marker, _toConsumableArray(this._popup));
        }
      }
    }, {
      key: "_removeMarker",
      value: function _removeMarker() {
        if (this._marker) {
          this._map.removeLayer(this._marker);

          delete this._marker;
        }
      }
    }, {
      key: "__updateMarker",
      value: function __updateMarker() {
        var color = this._getColor(val);

        var _ref2 = typeof color === 'string' ? (0, _palettes.cssToRGB)(color) : color,
            r = _ref2.r,
            g = _ref2.g,
            b = _ref2.b;

        this._marker.options.color = 'rgb(' + r + ',' + g + ',' + b + ')';
      }
    }, {
      key: "bindPopup",
      value: function bindPopup() {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        this._popup = args;

        if (this._marker) {
          var _this$_marker2;

          (_this$_marker2 = this._marker).bindPopup.apply(_this$_marker2, args);
        }
      }
    }, {
      key: "openPopup",
      value: function openPopup() {
        this._marker.openPopup();

        return this;
      }
    }, {
      key: "closePopup",
      value: function closePopup() {
        this._marker.closePopup();

        return this;
      }
    }, {
      key: "redraw",
      value: function redraw() {
        if (this._marker) {
          this.__updateMarker();

          this._marker.redraw();
        }
      }
    }]);

    return _class;
  }(base);
}