"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MultiPolygon = exports.DEFAULT_COLOR = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _covutils = require("covutils");

var _palettes = require("./palettes.js");

var _PaletteMixin2 = require("./PaletteMixin.js");

var _CoverageMixin = require("./CoverageMixin.js");

var _EventMixin = require("../util/EventMixin.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/** @ignore */
var DEFAULT_COLOR = 'black';
/**
 * The `click` event, signalling that a polygon has been clicked.
 * 
 * @typedef {L.MouseEvent} MultiPolygon#click
 * @property {number} index The axis index of the polygon that was clicked.
 * @property {L.LatLng} latlng The geographical point where the click event occured.
 */

/**
 * Renderer for Coverages and Domains conforming to CovJSON domain type `MultiPolygon`.
 *
 * @example
 * var cov = ... // get Coverage data
 * var layer = new C.MultiPolygon(cov, {
 *   parameter: 'salinity',
 *   defaultColor: 'black',
 *   palette: C.linearPalette(['#FFFFFF', '#000000'])
 * })
 * 
 * @see https://covjson.org/domain-types/#multipolygon
 * 
 * @emits {DataLayer#afterAdd} Layer is initialized and was added to the map
 * @emits {DataLayer#dataLoading} Data loading has started
 * @emits {DataLayer#dataLoad} Data loading has finished (also in case of errors)
 * @emits {DataLayer#error} Error when loading data
 * @emits {PaletteMixin#paletteChange} Palette has changed
 * @emits {PaletteMixin#paletteExtentChange} Palette extent has changed
 * @emits {MultiPolygon#click} when a polygon was clicked
 * 
 * @extends {L.Layer}
 * @extends {CoverageMixin}
 * @extends {PaletteMixin}
 * @implements {DataLayer}
 */

exports.DEFAULT_COLOR = DEFAULT_COLOR;

var MultiPolygon = /*#__PURE__*/function (_PaletteMixin) {
  _inherits(MultiPolygon, _PaletteMixin);

  var _super = _createSuper(MultiPolygon);

  /**
   * The key of the parameter to display can be given in the 'parameter' options property,
   * it will be ignored if the coverage data object is a Domain object.
   * 
   * @param {Coverage|Domain} cov The coverage or domain object to visualize.
   * @param {Object} [options] The options object.
   * @param {string} [options.parameter] The key of the parameter to display, not needed for domain objects.
   * @param {Palette} [options.palette] The initial color palette to use, the default depends on the parameter type.
   * @param {string} [options.paletteExtent='full'] The initial palette extent, either 'full' or specific: [-10,10].
   * @param {string} [options.defaultColor='black'] The color to use for missing data or if no parameter is set.
   */
  function MultiPolygon(cov, options) {
    var _this;

    _classCallCheck(this, MultiPolygon);

    _this = _super.call(this);

    if ((0, _covutils.isDomain)(cov)) {
      cov = (0, _covutils.fromDomain)(cov);
      options.parameter = cov.parameters.keys().next().value;
      delete options.keys;
    }

    if (!options.paletteExtent) {
      options.paletteExtent = 'full';
    }

    _leaflet.default.Util.setOptions(_assertThisInitialized(_this), options);

    _this._cov = cov;
    var paramKey = options.keys ? options.keys[0] : options.parameter;
    _this._param = paramKey ? cov.parameters.get(paramKey) : null;
    _this._defaultColor = options.defaultColor || DEFAULT_COLOR;
    return _this;
  }
  /**
   * @ignore
   * @override
   */


  _createClass(MultiPolygon, [{
    key: "onAdd",
    value: function onAdd(map) {
      var _this2 = this;

      this._map = map;
      this.load().then(function () {
        return _this2.initializePalette();
      }).then(function () {
        _this2._unproject();

        _this2._addPolygons();

        _this2._pointInPolygonPreprocess();

        _this2.fire('afterAdd');
      });
    }
  }, {
    key: "_unproject",
    value: function _unproject() {
      var unproject = this.projection.unproject;
      var axis = this.domain.axes.get('composite');
      var ix = axis.coordinates.indexOf(this._projX);
      var iy = axis.coordinates.indexOf(this._projY);
      this._polygonsLonLat = axis.values.map(function (polygon) {
        return polygon.map(function (ring) {
          return ring.map(function (coords) {
            var _unproject2 = unproject({
              x: coords[ix],
              y: coords[iy]
            }),
                lat = _unproject2.lat,
                lon = _unproject2.lon;

            return [lon, lat];
          });
        });
      });
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "onRemove",
    value: function onRemove(map) {
      this._removePolygons();
    }
    /**
     * Returns the geographic bounds of the coverage.
     * 
     * @return {L.LatLngBounds}
     */

  }, {
    key: "getBounds",
    value: function getBounds() {
      return this._geojson.getBounds();
    }
    /**
     * Returns the geographical center position of the coverage based on its bounding box.
     * 
     * @return {L.LatLng}
     */

  }, {
    key: "getLatLng",
    value: function getLatLng() {
      return this.getBounds().getCenter();
    }
    /**
     * The coverage object associated to this layer.
     * 
     * @type {Coverage}
     */

  }, {
    key: "coverage",
    get: function get() {
      return this._cov;
    }
    /**
     * The parameter that is visualized.
     * 
     * @type {Parameter}
     */

  }, {
    key: "parameter",
    get: function get() {
      return this._param;
    }
    /**
     * See {@link PaletteMixin}.
     * 
     * @ignore
     */

  }, {
    key: "computePaletteExtent",
    value: function computePaletteExtent(extent) {
      if (extent === 'full') {
        if (!this.parameter) {
          throw new Error('palette extent cannot be computed when no parameter has been chosen');
        }

        extent = (0, _covutils.minMaxOfRange)(this.range);
        extent = (0, _palettes.enlargeExtentIfEqual)(extent);
        return Promise.resolve(extent);
      } else {
        throw new Error('Unknown extent specification: ' + extent);
      }
    }
  }, {
    key: "_pointInPolygonPreprocess",
    value: function _pointInPolygonPreprocess() {
      var polygons = this._polygonsLonLat; // TODO we assume spherical coordinates for now

      var isCartesian = false; // A bit evil since this modifies in-place, but nothing bad should happen.

      polygons.forEach(function (p) {
        return (0, _covutils.ensureClockwisePolygon)(p, isCartesian);
      });
      this._pointInPolygons = (0, _covutils.getPointInPolygonsFn)(polygons);
    }
  }, {
    key: "_addPolygons",
    value: function _addPolygons() {
      var _this3 = this;

      var polygons = this._polygonsLonLat;
      var geojson = [];

      for (var i = 0; i < polygons.length; i++) {
        geojson.push({
          "type": "Feature",
          "properties": {
            "index": i,
            "color": this._getColor(this._getValue(i))
          },
          "geometry": {
            "type": "Polygon",
            "coordinates": polygons[i]
          }
        });
      }

      this._geojson = _leaflet.default.geoJson(geojson, {
        style: function style(feature) {
          return {
            color: feature.properties.color,
            fillOpacity: 1,
            stroke: false
          };
        },
        onEachFeature: function onEachFeature(feature, layer) {
          layer.on('click', function (e) {
            e.index = feature.properties.index;

            _this3.fire('click', e);
          });
        }
      });

      this._geojson.addTo(this._map);
    }
  }, {
    key: "_removePolygons",
    value: function _removePolygons() {
      this._map.removeLayer(this._geojson);

      delete this._geojson;
    }
  }, {
    key: "_getValue",
    value: function _getValue(index) {
      if (this._param) {
        return this.range.get({
          composite: index
        });
      }
    }
    /**
     * Return the displayed value at a given geographic position.
     * If out of bounds, then undefined is returned, otherwise a number or null (for no data).
     * 
     * @param {L.LatLng} latlng
     * @returns {number|null|undefined}
     */

  }, {
    key: "getValueAt",
    value: function getValueAt(latlng) {
      // TODO longitude wrapping
      var i = this._pointInPolygons([latlng.lng, latlng.lat]);

      if (i >= 0) {
        return this._getValue(i);
      }
    } // NOTE: this returns a string, not an {r,g,b} object as in other classes!

  }, {
    key: "_getColor",
    value: function _getColor(val) {
      if (val === null) {
        // no-data
        return this._defaultColor;
      } else if (val === undefined) {
        // not fixed to a param
        return this._defaultColor;
      } else {
        // use a palette
        var idx = this.getPaletteIndex(val);
        var _this$palette = this.palette,
            red = _this$palette.red,
            green = _this$palette.green,
            blue = _this$palette.blue;
        return "rgb(".concat(red[idx], ", ").concat(green[idx], ", ").concat(blue[idx], ")");
      }
    }
  }, {
    key: "_updatePolygons",
    value: function _updatePolygons() {
      this._removePolygons();

      this._addPolygons();
    }
    /**
     * Redraw the layer.
     */

  }, {
    key: "redraw",
    value: function redraw() {
      this._updatePolygons();

      this._geojson.redraw();
    }
  }]);

  return MultiPolygon;
}((0, _PaletteMixin2.PaletteMixin)((0, _CoverageMixin.CoverageMixin)(_leaflet.default.Layer)));

exports.MultiPolygon = MultiPolygon;