"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Point = exports.DEFAULT_COLOR = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _palettes = require("./palettes.js");

var _CoverageMixin = require("./CoverageMixin.js");

var _CircleMarkerMixin = require("./CircleMarkerMixin.js");

var _PaletteMixin2 = require("./PaletteMixin.js");

var _EventMixin = require("../util/EventMixin.js");

var _covutils = require("covutils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/** @ignore */
var DEFAULT_COLOR = 'black';
/**
 * The `click` event, signalling that the point has been clicked.
 * 
 * @typedef {L.MouseEvent} Point#click
 * @property {Coverage} coverage
 * @property {L.LatLng} latlng The geographical point where the click event occured.
 */

/**
 * Renderer for Coverages and Domains conforming to CovJSON domain type `Point`.
 * 
 * This will simply display a dot on the map and fire a click event when a user clicks on it.
 * The dot either has a defined standard color, or it uses a palette if a parameter is chosen.
 * 
 * @example
 * var cov = ... // get Coverage data
 * var layer = new C.Point(cov, {
 *   parameter: 'salinity',
 *   defaultColor: 'black',
 *   palette: C.linearPalette(['#FFFFFF', '#000000'])
 * })
 * 
 * @see https://covjson.org/domain-types/#point
 * 
 * @emits {DataLayer#afterAdd} Layer is initialized and was added to the map
 * @emits {DataLayer#dataLoading} Data loading has started
 * @emits {DataLayer#dataLoad} Data loading has finished (also in case of errors)
 * @emits {DataLayer#error} Error when loading data
 * @emits {PaletteMixin#paletteChange} Palette has changed
 * @emits {PaletteMixin#paletteExtentChange} Palette extent has changed
 * @emits {Point#click} when the point was clicked
 * 
 * @extends {L.Layer}
 * @extends {CoverageMixin}
 * @extends {CircleMarkerMixin}
 * @extends {PaletteMixin}
 * @implements {DataLayer}
 * @implements {PointDataLayer}
 */

exports.DEFAULT_COLOR = DEFAULT_COLOR;

var Point = /*#__PURE__*/function (_PaletteMixin) {
  _inherits(Point, _PaletteMixin);

  var _super = _createSuper(Point);

  /**
   * @param {Coverage|Domain} cov The coverage or domain object to visualize.
   * @param {Object} [options] The options object.
   * @param {string} [options.parameter] The key of the parameter to display, not needed for domain objects.
   * @param {Palette} [options.palette] The initial color palette to use, the default depends on the parameter type.
   * @param {string} [options.paletteExtent='full'] The initial palette extent, either 'full' or specific: [-10,10].
   * @param {string} [options.defaultColor='black'] The color to use for missing data or if no parameter is set.
   * @param {boolean} [options.showNoData=false] Whether to draw the point if there is no data.
   */
  function Point(cov, options) {
    var _this;

    _classCallCheck(this, Point);

    _this = _super.call(this);

    if ((0, _covutils.isDomain)(cov)) {
      cov = (0, _covutils.fromDomain)(cov);
      delete options.keys;
      options.parameter = cov.parameters.keys().next().value;
    }

    if (!options.paletteExtent) {
      options.paletteExtent = 'full';
    }

    _leaflet.default.Util.setOptions(_assertThisInitialized(_this), options);

    _this._cov = cov;
    var paramKey = options.keys ? options.keys[0] : options.parameter;
    _this._param = paramKey ? cov.parameters.get(paramKey) : null;
    _this._defaultColor = options.defaultColor || DEFAULT_COLOR;
    /** @ignore */

    _this.showNoData = options.showNoData; // if true, draw with default color

    return _this;
  }
  /**
   * @ignore
   * @override
   */


  _createClass(Point, [{
    key: "onAdd",
    value: function onAdd(map) {
      var _this2 = this;

      this._map = map;
      this.load().then(function () {
        return _this2.initializePalette();
      }).then(function () {
        _this2._addMarker();

        _this2.fire('afterAdd');
      });
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "onRemove",
    value: function onRemove() {
      this._removeMarker();
    }
    /**
     * Returns the geographic bounds of the coverage, which is a degenerate box collapsed to a point.
     * 
     * @return {L.LatLngBounds}
     */

  }, {
    key: "getBounds",
    value: function getBounds() {
      return _leaflet.default.latLngBounds([this.getLatLng()]);
    }
    /**
     * Returns the geographical position of the coverage.
     * 
     * @return {L.LatLng}
     */

  }, {
    key: "getLatLng",
    value: function getLatLng() {
      var x = this.domain.axes.get(this._projX).values[0];
      var y = this.domain.axes.get(this._projY).values[0];
      var latlng = this.projection.unproject({
        x: x,
        y: y
      });
      return _leaflet.default.latLng(latlng);
    }
    /**
     * The coverage object associated to this layer.
     * 
     * @type {Coverage}
     */

  }, {
    key: "coverage",
    get: function get() {
      return this._cov;
    }
    /**
     * The parameter that is visualized.
     * 
     * @type {Parameter}
     */

  }, {
    key: "parameter",
    get: function get() {
      return this._param;
    }
    /**
     * See {@link PaletteMixin}.
     * 
     * @ignore
     */

  }, {
    key: "computePaletteExtent",
    value: function computePaletteExtent(extent) {
      if (extent === 'full') {
        if (!this.parameter) {
          throw new Error('palette extent cannot be computed when no parameter has been chosen');
        }

        var val = this.getValue();
        extent = (0, _palettes.enlargeExtentIfEqual)([val, val]);
        return Promise.resolve(extent);
      } else {
        throw new Error('Unknown extent specification: ' + extent);
      }
    }
    /**
     * Return the displayed value (number, or null for no-data),
     * or undefined if no parameter is set.
     * 
     * @returns {number|null|undefined}
     */

  }, {
    key: "getValue",
    value: function getValue() {
      if (this._param) {
        return this.range.get({});
      }
    }
    /**
     * Return the displayed value if within the given distance of the reference point.
     * If out of bounds, then undefined is returned, otherwise a number or null (for no data).
     * 
     * @param {L.LatLng} latlng
     * @param {number} maxDistance Maximum distance in meters between both points.
     * @returns {number|null|undefined}
     */

  }, {
    key: "getValueAt",
    value: function getValueAt(latlng, maxDistance) {
      var point = this.getLatLng();

      if (point.distanceTo(latlng) <= maxDistance) {
        return this.getValue();
      }
    }
  }, {
    key: "_getColor",
    value: function _getColor(val) {
      if (val === null) {
        // no-data
        return this._defaultColor;
      } else if (val === undefined) {
        // not fixed to a param
        return this._defaultColor;
      } else {
        // use a palette
        var idx = this.getPaletteIndex(val);
        var _this$palette = this.palette,
            red = _this$palette.red,
            green = _this$palette.green,
            blue = _this$palette.blue;
        return {
          r: red[idx],
          g: green[idx],
          b: blue[idx]
        };
      }
    }
  }]);

  return Point;
}((0, _PaletteMixin2.PaletteMixin)((0, _CircleMarkerMixin.CircleMarkerMixin)((0, _CoverageMixin.CoverageMixin)(_leaflet.default.Layer))));

exports.Point = Point;