"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PointCollection = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _PaletteMixin2 = require("./PaletteMixin.js");

var _EventMixin = require("../util/EventMixin.js");

var _Point = require("./Point.js");

var _palettes = require("./palettes.js");

var _kdTree = require("../util/kdTree.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/**
 * A collection of points sharing the same parameters and coordinate referencing system.
 * 
 * @see https://covjson.org/domain-types/#point
 * 
 * @emits {DataLayer#afterAdd} Layer is initialized and was added to the map
 * @emits {DataLayer#error} Error when loading data
 * @emits {PaletteMixin#paletteChange} Palette has changed
 * @emits {PaletteMixin#paletteExtentChange} Palette extent has changed
 * @emits {Point#click} when the point was clicked
 * 
 * @extends {L.Layer}
 * @extends {PaletteMixin} 
 */
var PointCollection = /*#__PURE__*/function (_PaletteMixin) {
  _inherits(PointCollection, _PaletteMixin);

  var _super = _createSuper(PointCollection);

  /**
   * @param {CoverageCollection} covcoll The coverage collection to visualize.
   * @param {Object} [options] The options object.
   * @param {string} [options.parameter] The key of the parameter to display.
   * @param {Palette} [options.palette] The initial color palette to use, the default depends on the parameter type.
   * @param {string} [options.paletteExtent='full'] The initial palette extent, either 'full', 'fov', or specific: [-10,10].
   * @param {string} [options.defaultColor='black'] The color to use for missing data or if no parameter is set.
   * @param {class} [options.pointClass=Point] The {@link PointDataLayer} class to use for the individual points.
   * @param {function} [options.pointOptionsFn] A function that returns additional options to apply for each point class instance.  
   */
  function PointCollection(covcoll) {
    var _this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, PointCollection);

    _this = _super.call(this); // TODO how should we handle collection paging?

    if (!options.paletteExtent) {
      options.paletteExtent = 'full';
    }

    _leaflet.default.Util.setOptions(_assertThisInitialized(_this), options);

    _this._covcoll = covcoll;
    var paramKey = options.keys ? options.keys[0] : options.parameter;
    _this._param = paramKey ? covcoll.parameters.get(paramKey) : null;
    _this._defaultColor = options.defaultColor || _Point.DEFAULT_COLOR;
    _this._pointClass = options.pointClass || _Point.Point;
    _this._layerGroup = _leaflet.default.layerGroup();
    _this._layers = [];
    _this._kdtree = undefined;

    _this.on('paletteChange', function () {
      var _iterator = _createForOfIteratorHelper(_this._layers),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var layer = _step.value;
          layer.palette = _this.palette;
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    });

    _this.on('paletteExtentChange', function () {
      var _iterator2 = _createForOfIteratorHelper(_this._layers),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var layer = _step2.value;
          layer.paletteExtent = _this.paletteExtent;
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
    });

    return _this;
  }
  /**
   * @ignore
   * @override
   */


  _createClass(PointCollection, [{
    key: "onAdd",
    value: function onAdd(map) {
      this._map = map;
      this._layerLoadCount = 0;
      this._layerErrors = [];
      var options = {
        keys: this._param ? [this._param.key] : undefined,
        defaultColor: this._defaultColor,
        palette: this.palette,
        paletteExtent: this.paletteExtent
      };

      if (this.options.pointOptionsFn) {
        var opts = this.options.pointOptionsFn();

        for (var key in opts) {
          options[key] = opts[key];
        }
      }

      var _iterator3 = _createForOfIteratorHelper(this._covcoll.coverages),
          _step3;

      try {
        for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
          var cov = _step3.value;
          var layer = new this._pointClass(cov, options);

          this._attachListeners(layer, cov);

          this._layerGroup.addLayer(layer);

          this._layers.push(layer);

          layer.load();

          if (this._popupFn) {
            var popup = this._popupFn(layer.coverage);

            layer.bindPopup(popup);
          }
        }
      } catch (err) {
        _iterator3.e(err);
      } finally {
        _iterator3.f();
      }
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "onRemove",
    value: function onRemove(map) {
      map.removeLayer(this._layerGroup);
      this._layerGroup = _leaflet.default.layerGroup();
      this._layers = [];
    }
    /**
     * Binds a popup to each point instance.
     * 
     * @param {function(cov: Coverage):String|HTMLElement|L.Popup} fn Returns the popup for a given point coverage. 
     */

  }, {
    key: "bindPopupEach",
    value: function bindPopupEach(fn) {
      this._popupFn = fn;
    }
  }, {
    key: "_attachListeners",
    value: function _attachListeners(layer, cov) {
      var _this2 = this;

      layer.once('dataLoad', function () {
        ++_this2._layerLoadCount;

        _this2._fireIfOnAddDone();
      }).once('error', function (e) {
        _this2._layerErrors.push(e);
      }).on('click', function (e) {
        e.coverage = cov;

        _this2.fire('click', e);
      });
    }
  }, {
    key: "_fireIfOnAddDone",
    value: function _fireIfOnAddDone() {
      var _this3 = this;

      if (this._layerLoadCount === this._layers.length) {
        if (this._layerErrors.length > 0) {
          this.fire('error', {
            errors: this._layerErrors
          });
        } else {
          this._initKdtree();

          this.initializePalette().then(function () {
            _this3._layerGroup.addTo(_this3._map);

            _this3.fire('afterAdd');
          });
        }
      }
    }
  }, {
    key: "_initKdtree",
    value: function _initKdtree() {
      var points = this._layers.map(function (layer) {
        var point = layer.getLatLng();
        point.layer = layer;
        return point;
      });

      var distance = function distance(point1, point2) {
        return _leaflet.default.LatLng.prototype.distanceTo.call(point1, point2);
      };

      var dimensions = ['lat', 'lng'];
      this._kdtree = new _kdTree.kdTree(points, distance, dimensions);
    }
    /**
     * Returns the geographic bounds of the coverage collection.
     * 
     * @return {L.LatLngBounds}
     */

  }, {
    key: "getBounds",
    value: function getBounds() {
      return _leaflet.default.latLngBounds(this._layers.map(function (layer) {
        return layer.getLatLng();
      }));
    }
    /**
     * Return the displayed value of the point coverage closest to
     * the given position and within the given maximum distance.
     * If no coverage is found, undefined is returned, otherwise
     * a number or null (no-data).
     * 
     * @param {L.LatLng} latlng reference position
     * @param {number} maxDistance
     *   Maximum distance in meters that the point coverage may be
     *   apart from the given position.
     * @return {number|null|undefined}
     */

  }, {
    key: "getValueAt",
    value: function getValueAt(latlng, maxDistance) {
      var points = this._kdtree.nearest(latlng, 1, maxDistance);

      if (points.length > 0) {
        var point = points[0][0];
        var val = point.layer.getValue();
        return val;
      }
    }
    /**
     * The parameter that is visualized.
     * 
     * @type {Parameter}
     */

  }, {
    key: "parameter",
    get: function get() {
      return this._param;
    }
    /**
     * See {@link PaletteMixin}.
     * 
     * @ignore
     */

  }, {
    key: "computePaletteExtent",
    value: function computePaletteExtent(extent) {
      if (!this._param) {
        throw new Error('palette extent cannot be set when no parameter has been chosen');
      }

      var layers;

      if (extent === 'full') {
        layers = this._layers;
      } else if (extent === 'fov') {
        var bounds = this._map.getBounds();

        layers = this._layers.filter(function (layer) {
          return bounds.contains(layer.getLatLng());
        });
      } else {
        throw new Error('Unsupported: ' + extent);
      }

      var min = Infinity;
      var max = -Infinity;

      var _iterator4 = _createForOfIteratorHelper(layers),
          _step4;

      try {
        for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
          var layer = _step4.value;
          var val = layer.getValue();

          if (val != null) {
            min = Math.min(min, val);
            max = Math.max(max, val);
          }
        }
      } catch (err) {
        _iterator4.e(err);
      } finally {
        _iterator4.f();
      }

      extent = (0, _palettes.enlargeExtentIfEqual)([min, max]);
      return Promise.resolve(extent);
    }
    /**
     * Redraw each point layer.
     */

  }, {
    key: "redraw",
    value: function redraw() {
      var _iterator5 = _createForOfIteratorHelper(this._layers),
          _step5;

      try {
        for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
          var layer = _step5.value;
          layer.redraw();
        }
      } catch (err) {
        _iterator5.e(err);
      } finally {
        _iterator5.f();
      }
    }
  }]);

  return PointCollection;
}((0, _PaletteMixin2.PaletteMixin)(_leaflet.default.Layer));

exports.PointCollection = PointCollection;