"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PolygonSeries = void 0;

var _leaflet = _interopRequireDefault(require("leaflet"));

var _covutils = require("covutils");

var _palettes = require("./palettes.js");

var _CoverageMixin = require("./CoverageMixin.js");

var _EventMixin = require("../util/EventMixin.js");

var _PaletteMixin2 = require("./PaletteMixin.js");

var _Point = require("./Point.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

// TODO nearly identical to VerticalProfile

/**
 * Renderer for Coverages conforming to the CovJSON domain type `PolygonSeries`.
 * 
 * @example
 * var cov = ... // get Coverage data
 * var layer = new C.PolygonSeries(cov, {
 *   parameter: 'salinity',
 *   time: new Date('2015-01-01T12:00:00Z'),
 *   defaultColor: 'black',
 *   palette: C.linearPalette(['#FFFFFF', '#000000'])
 * })
 * 
 * @see https://covjson.org/domain-types/#polygonseries
 * 
 * @emits {DataLayer#afterAdd} Layer is initialized and was added to the map
 * @emits {DataLayer#dataLoading} Data loading has started
 * @emits {DataLayer#dataLoad} Data loading has finished (also in case of errors)
 * @emits {DataLayer#error} Error when loading data
 * @emits {DataLayer#axisChange} Axis coordinate has changed (e.axis === 'time')
 * @emits {PaletteMixin#paletteChange} Palette has changed
 * @emits {PaletteMixin#paletteExtentChange} Palette extent has changed
 * @emits {Point#click} when the polygon was clicked
 * 
 * @extends {L.Layer}
 * @extends {CoverageMixin}
 * @extends {PaletteMixin}
 * @implements {DataLayer}
 */
var PolygonSeries = /*#__PURE__*/function (_PaletteMixin) {
  _inherits(PolygonSeries, _PaletteMixin);

  var _super = _createSuper(PolygonSeries);

  /**
   * An optional time axis target value can be defined with the 'time' property.
   * The closest values on the time axis is chosen.
   * 
   * @param {Coverage|Domain} cov The coverage or domain object to visualize.
   * @param {Object} [options] The options object.
   * @param {string} [options.parameter] The key of the parameter to display, not needed for domain objects.
   * @param {Date} [options.time] The initial time step to display.
   * @param {Palette} [options.palette] The initial color palette to use, the default depends on the parameter type.
   * @param {string} [options.paletteExtent='full'] The initial palette extent, either 'full' or specific: [-10,10].
   * @param {string} [options.defaultColor='black'] The color to use for missing data or if no parameter is set.
   */
  function PolygonSeries(cov, options) {
    var _this;

    _classCallCheck(this, PolygonSeries);

    _this = _super.call(this);

    if ((0, _covutils.isDomain)(cov)) {
      cov = (0, _covutils.fromDomain)(cov);
      delete options.keys;
      options.parameter = cov.parameters.keys().next().value;
    }

    if (!options.paletteExtent) {
      options.paletteExtent = 'full';
    }

    _leaflet.default.Util.setOptions(_assertThisInitialized(_this), options);

    _this._cov = cov;
    var paramKey = options.keys ? options.keys[0] : options.parameter;
    _this._param = paramKey ? cov.parameters.get(paramKey) : null;
    _this._axesSubset = {
      t: {
        coordPref: options.time
      }
    };
    _this._defaultColor = options.defaultColor || _Point.DEFAULT_COLOR;
    return _this;
  }
  /**
   * @ignore
   * @override
   */


  _createClass(PolygonSeries, [{
    key: "onAdd",
    value: function onAdd(map) {
      var _this2 = this;

      this._map = map;
      this.load().then(function () {
        return _this2.initializePalette();
      }).then(function () {
        _this2._unproject();

        _this2._addPolygon();

        _this2._pointInPolygonPreprocess();

        _this2.fire('afterAdd');
      });
    }
  }, {
    key: "_unproject",
    value: function _unproject() {
      var unproject = this.projection.unproject;
      var axis = this.domain.axes.get('composite');
      var ix = axis.coordinates.indexOf(this._projX);
      var iy = axis.coordinates.indexOf(this._projY);
      this._polygonLonLat = axis.values[0].map(function (ring) {
        return ring.map(function (coords) {
          var _unproject2 = unproject({
            x: coords[ix],
            y: coords[iy]
          }),
              lat = _unproject2.lat,
              lon = _unproject2.lon;

          return [lon, lat];
        });
      });
    }
  }, {
    key: "_loadCoverageSubset",
    value: function _loadCoverageSubset() {
      var _this3 = this;

      // adapted from Grid.js
      var t = this._axesSubset.t;

      if (t.coordPref == undefined) {
        t.idx = t.coord = undefined;
      } else {
        var vals = this.domain.axes.get('t').values.map(function (v) {
          return (0, _covutils.asTime)(v);
        });
        t.idx = (0, _covutils.indexOfNearest)(vals, (0, _covutils.asTime)(t.coordPref));
        t.coord = vals[t.idx];
      }

      this.coverage.loadRange(this.parameter.key).then(function (range) {
        _this3.range = range;
      }); // Note that we don't subset the coverage currently, since there is no real need for it
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "onRemove",
    value: function onRemove() {
      this._removePolygon();
    }
    /**
     * Returns the geographic bounds of the coverage.
     * 
     * @return {L.LatLngBounds}
     */

  }, {
    key: "getBounds",
    value: function getBounds() {
      return this._geojson.getBounds();
    }
    /**
     * Returns the geographical center position of the coverage based on its bounding box.
     * 
     * @return {L.LatLng}
     */

  }, {
    key: "getLatLng",
    value: function getLatLng() {
      return this.getBounds().getCenter();
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "bindPopup",
    value: function bindPopup() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      this._popup = args;

      if (this._geojson) {
        var _this$_geojson;

        (_this$_geojson = this._geojson).bindPopup.apply(_this$_geojson, args);
      }

      return this;
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "openPopup",
    value: function openPopup() {
      this._geojson.openPopup();

      return this;
    }
    /**
     * @ignore
     * @override
     */

  }, {
    key: "closePopup",
    value: function closePopup() {
      this._geojson.closePopup();

      return this;
    }
    /**
     * The coverage object associated to this layer.
     * 
     * @type {Coverage}
     */

  }, {
    key: "coverage",
    get: function get() {
      return this._cov;
    }
    /**
     * The parameter that is visualized.
     * 
     * @type {Parameter}
     */

  }, {
    key: "parameter",
    get: function get() {
      return this._param;
    }
    /**
     * Sets the currently active time to the one closest to the given Date object.
     * 
     * @type {Date|undefined}
     */

  }, {
    key: "time",
    get:
    /**
     * The currently active time on the temporal axis as Date object, 
     * or undefined if no time is set.
     * 
     * @type {Date|undefined}
     */
    function get() {
      if (!this._axesSubset.t.coord) {
        return;
      }

      var time = this.domain.axes.get('t').values[this._axesSubset.t.idx];

      return new Date(time);
    }
    /**
     * The time slices that make up the coverage.
     * 
     * @type {Array<Date>}
     */
    ,
    set: function set(val) {
      var old = this.time;
      this._axesSubset.t.coordPref = val ? new Date((0, _covutils.asTime)(val)).toISOString() : undefined;

      this._loadCoverageSubset();

      if (old === this.time) return;
      this.redraw();
      this.fire('axisChange', {
        axis: 'time'
      });
    }
  }, {
    key: "timeSlices",
    get: function get() {
      return this.domain.axes.get('t').values.map(function (t) {
        return new Date(t);
      });
    }
    /**
     * See {@link PaletteMixin}.
     * 
     * @ignore
     */

  }, {
    key: "canUsePalette",
    value: function canUsePalette() {
      return this.time !== undefined;
    }
    /**
     * See {@link PaletteMixin}.
     * 
     * @ignore
     */

  }, {
    key: "computePaletteExtent",
    value: function computePaletteExtent(extent) {
      if (extent === 'full') {
        if (!this.parameter) {
          throw new Error('palette extent cannot be computed when no parameter has been chosen');
        }

        extent = (0, _covutils.minMaxOfRange)(this.range);
        extent = (0, _palettes.enlargeExtentIfEqual)(extent);
        return Promise.resolve(extent);
      } else {
        throw new Error('Unknown extent specification: ' + extent);
      }
    }
  }, {
    key: "_pointInPolygonPreprocess",
    value: function _pointInPolygonPreprocess() {
      var polygon = this._polygonLonLat; // TODO we assume spherical coordinates for now

      var isCartesian = false; // A bit evil since this modifies in-place, but nothing bad should happen.

      (0, _covutils.ensureClockwisePolygon)(polygon, isCartesian);
      var pointInPolygons = (0, _covutils.getPointInPolygonsFn)([polygon]);

      this._pointInPolygon = function (point) {
        return pointInPolygons(point) !== -1;
      };
    }
  }, {
    key: "_addPolygon",
    value: function _addPolygon() {
      var _this4 = this;

      var polygon = this._polygonLonLat;
      var geojson = {
        "type": "Feature",
        "properties": {
          "color": this._getColor(this.getValue())
        },
        "geometry": {
          "type": "Polygon",
          "coordinates": polygon
        }
      };
      this._geojson = _leaflet.default.geoJson(geojson, {
        style: function style(feature) {
          return {
            color: feature.properties.color,
            fillOpacity: 1,
            stroke: false
          };
        },
        onEachFeature: function onEachFeature(feature, layer) {
          layer.on('click', function (e) {
            return _this4.fire('click', e);
          });
        }
      });

      if (this._popup) {
        var _this$_geojson2;

        (_this$_geojson2 = this._geojson).bindPopup.apply(_this$_geojson2, _toConsumableArray(this._popup));
      }

      this._geojson.addTo(this._map);
    }
  }, {
    key: "_removePolygon",
    value: function _removePolygon() {
      this._map.removeLayer(this._geojson);

      delete this._geojson;
    }
    /**
     * Return the displayed value (number, or null for no-data),
     * or undefined if not fixed to a t-coordinate or parameter.
     * 
     * @returns {number|null|undefined}
     */

  }, {
    key: "getValue",
    value: function getValue() {
      if (this._param && this._axesSubset.t.coord !== undefined) {
        var val = this.range.get({
          t: this._axesSubset.t.idx
        });
        return val;
      }
    }
    /**
     * Return the displayed value at a given geographic position.
     * If out of bounds, then undefined is returned, otherwise a number or null (for no data).
     * 
     * @param {L.LatLng} latlng
     * @returns {number|null|undefined}
     */

  }, {
    key: "getValueAt",
    value: function getValueAt(latlng) {
      if (!latlng) throw new Error('latlng parameter missing'); // TODO longitude wrapping

      if (this._pointInPolygon([latlng.lng, latlng.lat])) {
        return this.getValue();
      }
    } // NOTE: this returns a string, not an {r,g,b} object as in other classes!

  }, {
    key: "_getColor",
    value: function _getColor(val) {
      if (val === null) {
        // no-data
        return this._defaultColor;
      } else if (val === undefined) {
        // not fixed to a param or z-coordinate
        return this._defaultColor;
      } else {
        // use a palette
        var idx = this.getPaletteIndex(val);
        console.log('getting value', val, idx);
        var _this$palette = this.palette,
            red = _this$palette.red,
            green = _this$palette.green,
            blue = _this$palette.blue;
        return "rgb(".concat(red[idx], ", ").concat(green[idx], ", ").concat(blue[idx], ")");
      }
    }
  }, {
    key: "_updatePolygon",
    value: function _updatePolygon() {
      this._removePolygon();

      this._addPolygon();
    }
    /**
     * Redraw the layer.
     */

  }, {
    key: "redraw",
    value: function redraw() {
      this._updatePolygon();

      this._geojson.setStyle({
        fillColor: this._getColor(this.getValue())
      });
    }
  }]);

  return PolygonSeries;
}((0, _PaletteMixin2.PaletteMixin)((0, _CoverageMixin.CoverageMixin)(_leaflet.default.Layer)));

exports.PolygonSeries = PolygonSeries;